/*
 * Copyright (C) 2015 Inteno Broadband Technology AB. All rights reserved.
 *
 * Author: Martin K. Schröder <mkschreder.uk@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

JUCI.app
.controller("SettingsUpgradeCtrl", function($scope, $uci, $config, $rpc, $tr, gettext, $juciDialog, $events, $file){
	$scope.sessionID = $rpc.$sid();
	$scope.usbFileName = "()";
	$scope.data = { upUrl: ""};
	$scope.usbUpgradeAvailable = false;
	$scope.upfile = {};
	var uploadFilename = "/tmp/firmware.bin";

	$uci.$sync("system").done(function(){
		$scope.system = $uci.system;
		$scope.$apply();
	});

	$rpc.$call("system", "board").done(function(info){
		$scope.board = info;
		$scope.$apply();
	});

	function confirmKeep(){
		var deferred = $.Deferred();
		$juciDialog.show(null, {
			title: $tr(gettext("Do you want to keep your configuration?")),
			content: $tr(gettext("If you choose yes then your configuration will be saved and restored after the upgrade has completed.")) + "<br />" +
				$tr(gettext("If you choose 'no' then all your current configuration will be reset to defaults.")),
			buttons: [
				{ label: $tr(gettext("Yes")), value: "yes", primary: true },
				{ label: $tr(gettext("No")), value: "no" },
				{ label: $tr(gettext("Abort")), value: "abort" }
			],
			on_button: function(btn, inst){
				if(btn.value == "yes"){
					inst.close();
					deferred.resolve("yes");
				}
				if(btn.value == "no"){
					inst.close();
					deferred.resolve("no");
				}
				if(btn.value == "abort"){
					inst.dismiss("abort");
					deferred.reject("abort");
				}
			}
		});

		return deferred.promise();
	}

	$scope.onDismissModal = function(){
		$scope.showUpgradeStatus = false;
	};

	$scope.onStartUpgrade = function(){
		if($scope.showUpgradeStatus)
			return;
		if(!($scope.upfile.files && $scope.upfile.files[0] && $scope.upfile.files[0].name) && !$scope.data.upUrl)
			return;

		confirmKeep().done(function(keep){
			startUpload(keep);
		});
	}

	$scope.fileChanged = function(){
		$scope.upfile = document.getElementById("imageFileSelector");
		$scope.$apply();
	}

	function upgradeFail(res) {
		$juciDialog.show(null, {
			title: $tr(gettext("Image check failed")),
			buttons: [{ label: $tr(gettext("OK")), value: "ok", primary: true }],
			on_button: function(btn, inst){
				inst.dismiss("ok");
			},
			content: ($tr(gettext("Error: ")) + ((res && res.stdout ? res.stdout : "Unknown")))
		});
	}

	function startUpload(res){
		var keep = (res == "yes")
		var upfile = $scope.upfile;
		if(!upfile.name || upfile.size < 1) return;
		$file.uploadFile(uploadFilename, upfile.files[0], function(progress, total){
			$scope.progress_byte = progress;
			$scope.progress_total = total;
			$scope.$apply();
		}).done(function(){
			$scope.progress_byte = $scope.progress_total;
			$scope.$apply();
			$rpc.$call("rpc-sys", "upgrade_test").done(function(res) {
				$scope.showUpgradeStatus = false;
				$scope.$apply();
				if (res && res.code == 0) {
					$rpc.$call("rpc-sys", "upgrade_start", { "keep": keep }).done(function(res) {
						window.location = "/reboot.html";
					}).fail(function(e) {
						$scope.error =  $tr(gettext("Unknown error"))+" ("+JSON.stringify(e)+")";
						$scope.$apply();
					});
				} else
					upgradeFail(res);
			}).fail(function(e) {
				$scope.error =  $tr(gettext("Unknown error"))+" ("+JSON.stringify(e)+")";
				$scope.$apply();
			});
		}).fail(function(e){
			$scope.error =  $tr(gettext("The server returned an error"))+" ("+JSON.stringify(e)+")";
			$scope.$apply();
		});
		$scope.progress_byte = 0;
		$scope.showUpgradeStatus = true;
		$scope.error = "";
	}
});
